@@ -90,6 +90,9 @@ ui_event (BonoboUIComponent           *component,
 {
 	UIListener *list;
 
+	g_return_if_fail (component != NULL);
+	g_return_if_fail (component->priv != NULL);
+
 	list = g_hash_table_lookup (component->priv->listeners, id);
 	if (list && list->cb)
 		list->cb (component, id, type,
@@ -114,6 +117,9 @@ impl_Bonobo_UIComponent_execVerb (PortableServer_Servant servant,
 
 	component = bonobo_ui_from_servant (servant);
 
+	g_return_if_fail (component != NULL);
+	g_return_if_fail (component->priv != NULL);
+
 	bonobo_object_ref (BONOBO_OBJECT (component));
 	
 /*	g_warning ("TESTME: Exec verb '%s'", cname);*/
@@ -176,6 +182,7 @@ bonobo_ui_component_add_verb_full (BonoboUIComponent  *component,
 
 	g_return_if_fail (cname != NULL);
 	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+	g_return_if_fail (component->priv != NULL);
 
 	priv = component->priv;
 
@@ -258,6 +265,9 @@ bonobo_ui_component_remove_verb (BonoboUIComponent  *component,
 {
 	RemoveInfo info;
 
+	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+	g_return_if_fail (component->priv != NULL);
+
 	memset (&info, 0, sizeof (info));
 
 	info.by_name = TRUE;
@@ -279,6 +289,9 @@ bonobo_ui_component_remove_verb_by_func (BonoboUIComponent  *component,
 {
 	RemoveInfo info;
 
+	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+	g_return_if_fail (component->priv != NULL);
+
 	memset (&info, 0, sizeof (info));
 
 	info.by_func = TRUE;
@@ -300,6 +313,9 @@ bonobo_ui_component_remove_verb_by_data (BonoboUIComponent  *component,
 {
 	RemoveInfo info;
 
+	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+	g_return_if_fail (component->priv != NULL);
+
 	memset (&info, 0, sizeof (info));
 
 	info.by_data = TRUE;
@@ -331,6 +347,7 @@ bonobo_ui_component_add_listener_full (BonoboUIComponent  *component,
 	g_return_if_fail (fn != NULL);
 	g_return_if_fail (id != NULL);
 	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+	g_return_if_fail (component->priv != NULL);
 
 	priv = component->priv;
 
@@ -403,6 +420,9 @@ bonobo_ui_component_remove_listener (BonoboUIComponent  *component,
 {
 	RemoveInfo info;
 
+	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+	g_return_if_fail (component->priv != NULL);
+
 	memset (&info, 0, sizeof (info));
 
 	info.by_name = TRUE;
@@ -424,6 +444,9 @@ bonobo_ui_component_remove_listener_by_func (BonoboUIComponent  *component,
 {
 	RemoveInfo info;
 
+	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+	g_return_if_fail (component->priv != NULL);
+
 	memset (&info, 0, sizeof (info));
 
 	info.by_func = TRUE;
@@ -445,6 +468,9 @@ bonobo_ui_component_remove_listener_by_data (BonoboUIComponent  *component,
 {
 	RemoveInfo info;
 
+	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+	g_return_if_fail (component->priv != NULL);
+
 	memset (&info, 0, sizeof (info));
 
 	info.by_data = TRUE;
@@ -619,6 +645,7 @@ impl_xml_set (BonoboUIComponent  *component,
 
 	g_return_if_fail (xml != NULL);
 	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+	g_return_if_fail (component->priv != NULL);
 	container = component->priv->container;
 	g_return_if_fail (container != CORBA_OBJECT_NIL);
 
@@ -635,7 +662,7 @@ impl_xml_set (BonoboUIComponent  *component,
 	name = component->priv->name ? component->priv->name : "";
 
 	Bonobo_UIContainer_setNode (container, path, xml,
-				     name, real_ev);
+				    name, real_ev);
 
 	if (BONOBO_EX (real_ev) && !ev)
 		g_warning ("Serious exception on node_set '$%s' of '%s' to '%s'",
@@ -741,6 +768,7 @@ impl_xml_get (BonoboUIComponent *component,
 	Bonobo_UIContainer container;
 
 	g_return_val_if_fail (BONOBO_IS_UI_COMPONENT (component), NULL);
+	g_return_val_if_fail (component->priv != NULL, NULL);
 	container = component->priv->container;
 	g_return_val_if_fail (container != CORBA_OBJECT_NIL, NULL);
 
@@ -757,9 +785,16 @@ impl_xml_get (BonoboUIComponent *component,
 		if (!ev)
 			g_warning ("Serious exception getting node '%s' '$%s'",
 				   path, bonobo_exception_get_text (real_ev));
+
+		if (!ev)
+			CORBA_exception_free (&tmp_ev);
+
 		return NULL;
 	}
 
+	if (!ev)
+		CORBA_exception_free (&tmp_ev);
+
 	return xml;
 }
 
@@ -829,6 +864,7 @@ impl_xml_rm (BonoboUIComponent  *component,
 	Bonobo_UIContainer container;
 
 	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+	g_return_if_fail (component->priv != NULL);
 	container = component->priv->container;
 	g_return_if_fail (container != CORBA_OBJECT_NIL);
 
@@ -875,6 +911,7 @@ bonobo_ui_component_object_set (BonoboUIComponent  *component,
 	Bonobo_UIContainer container;
 
 	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+	g_return_if_fail (component->priv != NULL);
 	container = component->priv->container;
 	g_return_if_fail (container != CORBA_OBJECT_NIL);
 
@@ -918,6 +955,8 @@ bonobo_ui_component_object_get (BonoboUIComponent  *component,
 
 	g_return_val_if_fail (BONOBO_IS_UI_COMPONENT (component),
 			      CORBA_OBJECT_NIL);
+	g_return_val_if_fail (component->priv != NULL,
+			      CORBA_OBJECT_NIL);
 	container = component->priv->container;
 	g_return_val_if_fail (container != CORBA_OBJECT_NIL,
 			      CORBA_OBJECT_NIL);
@@ -1014,6 +1053,7 @@ impl_freeze (BonoboUIComponent *component,
 	Bonobo_UIContainer container;
 
 	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+	g_return_if_fail (component->priv != NULL);
 	container = component->priv->container;
 	g_return_if_fail (container != CORBA_OBJECT_NIL);
 
@@ -1061,6 +1101,7 @@ impl_thaw (BonoboUIComponent *component,
 	Bonobo_UIContainer container;
 
 	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+	g_return_if_fail (component->priv != NULL);
 	container = component->priv->container;
 	g_return_if_fail (container != CORBA_OBJECT_NIL);
 
@@ -1181,9 +1222,10 @@ impl_get_prop (BonoboUIComponent *component,
 	char *full_path;
 	xmlChar *ans;
 	gchar   *ret;
-        CORBA_Environment hack_ev;
+        CORBA_Environment *real_ev, tmp_ev;
 
 	g_return_val_if_fail (BONOBO_IS_UI_COMPONENT (component), NULL);
+	g_return_val_if_fail (component->priv != NULL, NULL);
 
 	full_path = alloca (strlen (path) + 1 + strlen (prop) + 1);
 	strcpy (full_path, path);
@@ -1192,13 +1234,15 @@ impl_get_prop (BonoboUIComponent *component,
 
 	bonobo_object_ref (BONOBO_OBJECT (component));
 
-	if (!opt_ev) {
+	if (opt_ev)
+		real_ev = opt_ev;
+	else {
 		/* Hack to avoid warnings for reading non-existant properties */
-		CORBA_exception_init (&hack_ev);
-		opt_ev = &hack_ev;
+		CORBA_exception_init (&tmp_ev);
+		real_ev = &tmp_ev;
 	}
 
-	ans = bonobo_ui_component_get (component, full_path, FALSE, opt_ev);
+	ans = bonobo_ui_component_get (component, full_path, FALSE, real_ev);
 
 	if (ans) {
 		ret = g_strdup (ans);
@@ -1207,6 +1251,9 @@ impl_get_prop (BonoboUIComponent *component,
 		ret = NULL;
 
 	bonobo_object_unref (BONOBO_OBJECT (component));
+
+	if (!opt_ev)
+		CORBA_exception_free (&tmp_ev);
 	
 	return ret;
 }
@@ -1237,6 +1284,7 @@ impl_exists (BonoboUIComponent *component,
 	CORBA_Environment *real_ev, tmp_ev;
 
 	g_return_val_if_fail (BONOBO_IS_UI_COMPONENT (component), FALSE);
+	g_return_val_if_fail (component->priv != NULL, FALSE);
 	container = component->priv->container;
 	g_return_val_if_fail (container != CORBA_OBJECT_NIL, FALSE);
 
@@ -1305,6 +1353,7 @@ void
 bonobo_ui_component_unset_container (BonoboUIComponent *component)
 {
 	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+	g_return_if_fail (component->priv != NULL);
 
 	bonobo_object_ref (BONOBO_OBJECT (component));
 
@@ -1351,6 +1400,7 @@ bonobo_ui_component_set_container (BonoboUIComponent *component,
 	Bonobo_UIContainer ref_cont;
 
 	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+	g_return_if_fail (component->priv != NULL);
 
 	bonobo_object_ref (BONOBO_OBJECT (component));
 
@@ -1397,6 +1447,8 @@ bonobo_ui_component_get_container (BonoboUIComponent *component)
 {
 	g_return_val_if_fail (BONOBO_IS_UI_COMPONENT (component),
 			      CORBA_OBJECT_NIL);
+	g_return_val_if_fail (component->priv != NULL,
+			      CORBA_OBJECT_NIL);
 	
 	return component->priv->container;
 }
