@@ -1182,6 +1182,8 @@ bonobo_ui_util_fixup_icons (BonoboUINode *node)
 			bonobo_ui_node_set_attr (node, "pixtype", "pixbuf");
 			bonobo_ui_node_set_attr (node, "pixname", xml);
 			g_free (xml);
+
+			gdk_pixbuf_unref (pixbuf);
 		}
 
 		bonobo_ui_node_free_string (txt);
@@ -1191,7 +1193,6 @@ bonobo_ui_util_fixup_icons (BonoboUINode *node)
 		bonobo_ui_util_fixup_icons (l);
 }
 
-
 /**
  * bonobo_ui_util_new_ui:
  * @component: The component help callback should be on
@@ -1225,6 +1226,59 @@ bonobo_ui_util_new_ui (BonoboUIComponent *component,
 	return node;
 }
 
+typedef struct {
+	char *file_name;
+	char *app_datadir;
+	char *app_name;
+	char *tree;
+} BonoboUINodeCacheEntry;
+
+static guint
+node_hash (gconstpointer key)
+{
+	BonoboUINodeCacheEntry *entry = (BonoboUINodeCacheEntry *)key;
+	/* Ignore the app_datadir in the hash, since that 
+	   is also in file_name (always?) */
+	return g_str_hash (entry->file_name) ^ g_str_hash (entry->app_name);
+}
+
+static gint
+node_equal (gconstpointer a,
+	    gconstpointer b)
+{
+	BonoboUINodeCacheEntry *entry_a = (BonoboUINodeCacheEntry *)a;
+	BonoboUINodeCacheEntry *entry_b = (BonoboUINodeCacheEntry *)b;
+
+	return  (strcmp (entry_a->file_name, entry_b->file_name) == 0) &&
+		(strcmp (entry_a->app_name, entry_b->app_name) == 0) &&
+		((entry_a->app_datadir == NULL && entry_b->app_datadir == NULL) ||
+		 (entry_a->app_datadir != NULL && entry_b->app_datadir != NULL &&
+		  (strcmp (entry_a->app_datadir, entry_b->app_datadir) == 0)));
+}
+
+static GHashTable *loaded_node_cache = NULL;
+
+static void
+free_node_cache_entry (BonoboUINodeCacheEntry *entry)
+{
+	g_free (entry->file_name);
+	g_free (entry->app_datadir);
+	g_free (entry->app_name);
+	g_free (entry->tree);
+	g_free (entry);
+}
+
+static void
+free_loaded_node_cache (void)
+{
+	if (loaded_node_cache) {
+		g_hash_table_foreach (loaded_node_cache,
+				      (GHFunc) free_node_cache_entry,
+				      NULL);
+		g_hash_table_destroy (loaded_node_cache);
+	}
+}
+
 /**
  * bonobo_ui_util_set_ui:
  * @component: the component
@@ -1244,7 +1298,15 @@ bonobo_ui_util_set_ui (BonoboUIComponent *component,
 		       const char        *app_name)
 {
 	char *fname;
-	BonoboUINode *ui;
+	char *ui;
+	BonoboUINodeCacheEntry entry, *cached;
+	BonoboUINode *node;
+	
+	if (!loaded_node_cache) {
+		loaded_node_cache = g_hash_table_new (node_hash,
+						      node_equal);
+		g_atexit (free_loaded_node_cache);
+	}
 
 	if (bonobo_ui_component_get_container (component) == CORBA_OBJECT_NIL) {
 		g_warning ("Component must be associated with a container first "
@@ -1258,15 +1320,34 @@ bonobo_ui_util_set_ui (BonoboUIComponent *component,
 		return;
 	}
 
-	ui = bonobo_ui_util_new_ui (component, fname, app_datadir, app_name);
+	/* FIXME: May want to stat the file to see if it changed? */
+	entry.file_name = (char *)fname;
+	entry.app_datadir = (char *)app_datadir;
+	entry.app_name = (char *)app_name;
+	
+	cached = g_hash_table_lookup (loaded_node_cache, &entry);
+	if (cached) 
+		ui = cached->tree;
+	else {
+		node = bonobo_ui_util_new_ui (component, fname, app_datadir, app_name);
+		ui = bonobo_ui_node_to_string (node, TRUE);
+		bonobo_ui_node_free (node);
+		
+		cached = g_new (BonoboUINodeCacheEntry, 1);
+		
+		cached->file_name = g_strdup (fname);
+		cached->app_datadir = g_strdup (app_datadir);
+		cached->app_name = g_strdup (app_name);
+		cached->tree = ui;
+		
+		g_hash_table_insert (loaded_node_cache,
+				     cached, cached);
+	}
 	
 	if (ui)
-		bonobo_ui_component_set_tree (
-			component, "/", ui, NULL);
+		bonobo_ui_component_set (component, "/", ui, NULL);
 	
 	g_free (fname);
-	bonobo_ui_node_free (ui);
-	/* FIXME: we could be caching the tree here */
 }
 
 /*
@@ -1622,95 +1703,6 @@ bonobo_ui_util_set_pixbuf (BonoboUIComponent *component,
 	g_free (parent_path);
 }
 
-/*
- *   Evil routines; see invocation above for rational,
- * use a subset of utf-8 encoding to save extra library
- * dependencies and yet more complexity.
- */
-#ifdef IF_ONLY_LIBXML1_COULD_COPE_WITH_UTF8
-
-/**
- * bonobo_ui_util_encode_str:
- * @str: the string
- * 
- * This function is needed to encode labels that contain
- * either UTF-8 or other 8 bit charsets. This is due to
- * severe libxml1 breakage. If you want to do manual
- * label setting you need this function.
- * 
- * Return value: the mangled (hex) text.
- **/
-char *
-bonobo_ui_util_encode_str (const char *str)
-{
-	const guint8 *a;
-	guint8       *b, *ret;
-	int           encoded_len;
-
-	if (!str)
-		return NULL;
-
-	return g_strdup (str);
-
-	encoded_len = 0;
-	for (a = str; *a; a++)
-		encoded_len += *a > 0x7f ? 2 : 1;
-
-	ret = g_malloc (encoded_len + 1);
-
-	b = ret;
-	for (a = str; *a; a++) {
-		if (*a > 0x7f) {
-			*b++ = 0xc0 | ((*a) >> 6);
-			*b++ = 0x80 | ((*a) & 0x3f);
-		} else
-			*b++ = *a;
-	}
-	*b = '\0';
-
-	return ret;
-}
-
-/**
- * bonobo_ui_util_decode_str:
- * @str: the string
- * @err: a pointer to an error flag
- * 
- * This provides the inverse mapping for bonobo_ui_util_encode_str
- * 
- * Return value: the decoded string or NULL on failure.
- **/
-char *
-bonobo_ui_util_decode_str (const char *str, gboolean *err)
-{
-	const guint8 *a;
-	guint8       *b, *ret;
-
-	g_return_val_if_fail (err != NULL, NULL);
-	*err = FALSE;
-
-	if (!str)
-		return NULL;
-
-	return g_strdup (str);
-
-	ret = g_malloc (strlen (str) + 1);
-
-	b = ret;
-	for (a = str; *a; a++) {
-		if (*a > 0x7f && *(a + 1) != '\0') {
-			*b++ = (*a << 6) | (*(a + 1) & 0x3f);
-			a++;
-		} else
-			*b++ = *a;
-	}
-	*b = '\0';
-
-	return ret;
-}
-
-#else
-
 /*
  *  And here we start to have a major headache.
  * Not only does libxml1 not support utf-8 encoding
@@ -1776,4 +1768,3 @@ bonobo_ui_util_decode_str (const char *str, gboolean *err)
 
 	return ret;
 }
-#endif
